import * as moment_ from "moment";
import { Employee } from "../employee/employee";
import { BridgeDto } from "./bridge-dto";

const moment = moment_; // @HACK Necessary because of https://github.com/rollup/rollup/issues/670

export class Bridge {
    public id?: string;
    public employee: Employee;
    public originDate: Date;
    public destinationDate: Date;
    public createdBy?: string;
    public createdDate?: Date;
    public lastModifiedBy?: string;
    public lastModifiedDate?: Date;
    public custom?: any;

    public static fromDto(bridgeDto: BridgeDto, originEntity?: string): Bridge {
        const model: any = { ...bridgeDto };

        model.originDate = model.originDate && moment(model.originDate).toDate();
        model.destinationDate = model.destinationDate && moment(model.destinationDate).toDate();
        model.createdDate = model.createdDate && moment(model.createdDate).toDate();
        model.lastModifiedDate = model.lastModifiedDate && moment(model.lastModifiedDate).toDate();

        const lookupSeparator = " - ";
        const displayFields = [
            "id",
            "employee.code",
            "employee.person.nomPes",
            "originDate",
            "destinationDate",
            "createdBy",
            "createdDate",
            "lastModifiedBy",
            "lastModifiedDate",
        ];

        model.label = displayFields.map(field => model[field]).join(lookupSeparator);

        if (originEntity !== "Employee" && model.employee) model.employee = Employee.fromDto(model.employee, "Bridge");

        return model as Bridge;
    }

    public static toDto(bridge: Bridge, originEntity?: string): BridgeDto {
        const dto: any = { ...bridge };

        dto.originDate = dto.originDate && moment(dto.originDate).format("YYYY-MM-DD");
        dto.destinationDate = dto.destinationDate && moment(dto.destinationDate).format("YYYY-MM-DD");
        dto.createdDate = dto.createdDate && moment(dto.createdDate).format();
        dto.lastModifiedDate = dto.lastModifiedDate && moment(dto.lastModifiedDate).format();

        delete dto.label;

        if (originEntity !== "Employee" && dto.employee) dto.employee = Employee.toDto(dto.employee, "Bridge");

        return dto;
    }
}