import * as moment_ from "moment";
import { TimeEvaluationDefinition } from "../time-evaluation-definition/time-evaluation-definition";
import { TimeEvaluationSituationDefinition } from "../time-evaluation-situation-definition/time-evaluation-situation-definition";
import { CompanyClockingEventIndicationsDto } from "./company-clocking-event-indications-dto";

const moment = moment_; // @HACK Necessary because of https://github.com/rollup/rollup/issues/670

export class CompanyClockingEventIndications {
    public id?: string;
    public timeEvaluationDefinition: TimeEvaluationDefinition;
    public timeEvaluationSituationDefinition: TimeEvaluationSituationDefinition;
    public createdBy?: string;
    public createdDate?: Date;
    public lastModifiedBy?: string;
    public lastModifiedDate?: Date;
    public custom?: any;

    public static fromDto(
        companyClockingEventIndicationsDto: CompanyClockingEventIndicationsDto,
        originEntity?: string
    ): CompanyClockingEventIndications {
        const model: any = { ...companyClockingEventIndicationsDto };

        model.createdDate = model.createdDate && moment(model.createdDate).toDate();
        model.lastModifiedDate = model.lastModifiedDate && moment(model.lastModifiedDate).toDate();

        const lookupSeparator = " - ";
        const displayFields = [
            "timeEvaluationDefinition.code",
            "timeEvaluationDefinition.name",
            "timeEvaluationSituationDefinition.code",
            "timeEvaluationSituationDefinition.name",
        ];

        model.label = displayFields.map(field => model[field]).join(lookupSeparator);

        if (originEntity !== "TimeEvaluationDefinition" && model.timeEvaluationDefinition)
            model.timeEvaluationDefinition = TimeEvaluationDefinition.fromDto(
                model.timeEvaluationDefinition,
                "CompanyClockingEventIndications"
            );
        if (originEntity !== "TimeEvaluationSituationDefinition" && model.timeEvaluationSituationDefinition)
            model.timeEvaluationSituationDefinition = TimeEvaluationSituationDefinition.fromDto(
                model.timeEvaluationSituationDefinition,
                "CompanyClockingEventIndications"
            );

        return model as CompanyClockingEventIndications;
    }

    public static toDto(
        companyClockingEventIndications: CompanyClockingEventIndications,
        originEntity?: string
    ): CompanyClockingEventIndicationsDto {
        const dto: any = { ...companyClockingEventIndications };

        dto.createdDate = dto.createdDate && moment(dto.createdDate).format();
        dto.lastModifiedDate = dto.lastModifiedDate && moment(dto.lastModifiedDate).format();

        delete dto.label;

        if (originEntity !== "TimeEvaluationDefinition" && dto.timeEvaluationDefinition)
            dto.timeEvaluationDefinition = TimeEvaluationDefinition.toDto(dto.timeEvaluationDefinition, "CompanyClockingEventIndications");
        if (originEntity !== "TimeEvaluationSituationDefinition" && dto.timeEvaluationSituationDefinition)
            dto.timeEvaluationSituationDefinition = TimeEvaluationSituationDefinition.toDto(
                dto.timeEvaluationSituationDefinition,
                "CompanyClockingEventIndications"
            );

        return dto;
    }

    /*{CA:CLASS_METHODS:START}*/
    /*{CA:CLASS_METHODS:END}*/
}

/*{CA:FILE_CONTENTS:START}*/
/*{CA:FILE_CONTENTS:END}*/
