import * as moment_ from "moment";
import { E001Pes } from "../e-001-pes/e-001-pes";
import { EmployeeDto } from "./employee-dto";

const moment = moment_; // @HACK Necessary because of https://github.com/rollup/rollup/issues/670

export class Employee {
    public id?: string;
    public code: number;
    public person?: E001Pes;
    public employer: { id: string };
    public hireDate?: Date;
    public dismissalDate?: Date;
    public deleted?: boolean = false;
    public extIntSts?: string;
    public extIntMsg?: string;
    public createdBy?: string;
    public createdDate?: Date;
    public lastModifiedBy?: string;
    public lastModifiedDate?: Date;
    public custom?: any;

    public static fromDto(employeeDto: EmployeeDto, originEntity?: string): Employee {
        const model: any = { ...employeeDto };

        model.hireDate = model.hireDate && moment(model.hireDate).toDate();
        model.dismissalDate = model.dismissalDate && moment(model.dismissalDate).toDate();
        model.createdDate = model.createdDate && moment(model.createdDate).toDate();
        model.lastModifiedDate = model.lastModifiedDate && moment(model.lastModifiedDate).toDate();

        const lookupSeparator = " - ";
        const displayFields = [
            "id",
            "code",
            "person.nomPes",
            "hireDate",
            "dismissalDate",
            "deleted",
            "extIntSts",
            "extIntMsg",
            "createdBy",
            "createdDate",
            "lastModifiedBy",
            "lastModifiedDate",
        ];

        model.label = displayFields.map(field => model[field]).join(lookupSeparator);

        if (originEntity !== "E001Pes" && model.person) model.person = E001Pes.fromDto(model.person, "Employee");

        return model as Employee;
    }

    public static toDto(employee: Employee, originEntity?: string): EmployeeDto {
        const dto: any = { ...employee };

        dto.hireDate = dto.hireDate && moment(dto.hireDate).format("YYYY-MM-DD");
        dto.dismissalDate = dto.dismissalDate && moment(dto.dismissalDate).format("YYYY-MM-DD");
        dto.createdDate = dto.createdDate && moment(dto.createdDate).format();
        dto.lastModifiedDate = dto.lastModifiedDate && moment(dto.lastModifiedDate).format();

        delete dto.label;

        if (originEntity !== "E001Pes" && dto.person) dto.person = E001Pes.toDto(dto.person, "Employee");

        return dto;
    }
}
