import * as moment_ from "moment";
import { ClockingEventToleranceType } from "../../enums/clocking-event-tolerance-type";
import { ConsiderClockingEventIntervalType } from "../../enums/consider-clocking-event-interval-type";
import { VerifiedDaysControlType } from "../../enums/verified-days-control-type";
import { RetroactiveAdjustmentType } from "../../enums/retroactive-adjustment-type";
import { Situation } from "../situation/situation";
import { TimeEvaluationDefinitionDto } from "./time-evaluation-definition-dto";
/*{CA:PACKAGE_IMPORTS:START}*/
/*{CA:PACKAGE_IMPORTS:END}*/

/*{CA:PROJECT_IMPORTS:START}*/
const moment = moment_; // @HACK Necessary because of https://github.com/rollup/rollup/issues/670
/*{CA:PROJECT_IMPORTS:END}*/

export class TimeEvaluationDefinition {
    public id?: string;
    public code: number;
    public name: string;
    public useWorkscheduleDayOff?: boolean;
    public verifiedDaysControl?: VerifiedDaysControlType;
    public controlAdjustmentReason?: boolean;
    public controlObservation?: boolean;
    public daysResume?: number;
    public clockingEventTolerance?: ClockingEventToleranceType;
    public retroactiveAdjustment?: RetroactiveAdjustmentType;
    public evaluateIncompleteJourney?: boolean;
    public incompleteJourneySituation?: Situation;
    public numberOfDaysAdjustment?: number;
    public calculateHolidayAbsenceRotationWorkshift?: boolean;
    public considerClockingEventInterval?: ConsiderClockingEventIntervalType;
    public clockingEventInterval?: number;
    public schedulingOverlapsHoliday?: boolean;
    public limitDaysBackwardScheduling?: number;
    public bridgeOverlapsWorkscheduleChange?: boolean;
    public allowClockingEventModificationBeforeTransference?: boolean;
    public calculateReadiness?: boolean;
    public discountHoursWorkedReadiness?: boolean;
    public expirationDays?: number;
    public limitOfDaysAfterPointClosing?: number;
    public blockPendingAfterExpires?: boolean;
    public allowsTakeLedgerPending?: boolean;
    public allowsCompleteUntreatedPending?: boolean;
    public createdBy?: string;
    public createdDate?: Date;
    public lastModifiedBy?: string;
    public lastModifiedDate?: Date;
    public custom?: any;
    /*{CA:CLASS_ATTRIBUTES:START}*/
    /*{CA:CLASS_ATTRIBUTES:END}*/

    public static fromDto(timeEvaluationDefinitionDto: TimeEvaluationDefinitionDto, originEntity?: string): TimeEvaluationDefinition {
        const model: any = { ...timeEvaluationDefinitionDto };

        model.createdDate = model.createdDate && moment(model.createdDate).toDate();
        model.lastModifiedDate = model.lastModifiedDate && moment(model.lastModifiedDate).toDate();

        const lookupSeparator = " - ";
        const displayFields = ["code", "name"];

        model.label = displayFields.map(field => model[field]).join(lookupSeparator);

        if (originEntity !== "Situation" && model.incompleteJourneySituation)
            model.incompleteJourneySituation = Situation.fromDto(model.incompleteJourneySituation, "TimeEvaluationDefinition");

        return model as TimeEvaluationDefinition;
    }

    public static toDto(timeEvaluationDefinition: TimeEvaluationDefinition, originEntity?: string): TimeEvaluationDefinitionDto {
        const dto: any = { ...timeEvaluationDefinition };

        dto.createdDate = dto.createdDate && moment(dto.createdDate).format();
        dto.lastModifiedDate = dto.lastModifiedDate && moment(dto.lastModifiedDate).format();

        delete dto.label;

        if (originEntity !== "Situation" && dto.incompleteJourneySituation)
            dto.incompleteJourneySituation = Situation.toDto(dto.incompleteJourneySituation, "TimeEvaluationDefinition");

        return dto;
    }

    /*{CA:CLASS_METHODS:START}*/
    /*{CA:CLASS_METHODS:END}*/
}

/*{CA:FILE_CONTENTS:START}*/
/*{CA:FILE_CONTENTS:END}*/
