import { AbstractControl, ValidationErrors } from "@angular/forms";
import * as moment_ from "moment";

const moment = moment_; // @HACK Necessary because of https://github.com/rollup/rollup/issues/670

export function startDateTimeIsBeforeEndDateTimeValidator(startDateTime: AbstractControl, endDateTime: AbstractControl): ValidationErrors | null {
    if (startDateTime && endDateTime && startDateTime.value && endDateTime.value) {
        const isInvalid = moment(startDateTime.value).seconds(0).isSameOrAfter(moment(endDateTime.value).seconds(0)) ? { startDateTimeIsBeforeEndDateTime: { value: [endDateTime.value, startDateTime.value] } } : null;
        if (isInvalid) {
            startDateTime.markAsDirty();
            endDateTime.markAsDirty();
            endDateTime.setErrors(isInvalid);
            startDateTime.setErrors(isInvalid);
            return isInvalid;
        }
        removeErrorFromField(endDateTime, 'startDateTimeIsBeforeEndDateTime');
        removeErrorFromField(startDateTime, 'startDateTimeIsBeforeEndDateTime');
    } else if (startDateTime && startDateTime.value && !(endDateTime && endDateTime.value)) {
        removeErrorFromField(startDateTime, 'startDateTimeIsBeforeEndDateTime')
    }
    else if (!(startDateTime && startDateTime.value) && endDateTime && endDateTime.value) {
        removeErrorFromField(endDateTime, 'startDateTimeIsBeforeEndDateTime')
    }

    return null;
};

export function startDateIsBeforeOrEqualToEndDateValidator(startDate: AbstractControl, endDate: AbstractControl): ValidationErrors | null {
    if(startDate && endDate && startDate.value && endDate.value) {
        const forbidden = startDate.value > endDate.value ? { startDateIsBeforeEndDate: { value: endDate.value } } : null;
        if(forbidden) {
            startDate.markAsDirty();
            endDate.markAsDirty();
            startDate.setErrors(forbidden);
            endDate.setErrors(forbidden);
            return forbidden;
        }
        removeErrorFromField(endDate, 'startDateIsBeforeEndDate');
        removeErrorFromField(startDate, 'startDateIsBeforeEndDate');
    } else if (startDate && startDate.value && !(endDate && endDate.value)) {
        removeErrorFromField(startDate, 'startDateIsBeforeEndDate')
    }
    else if (!(startDate && startDate.value) && endDate && endDate.value) {
        removeErrorFromField(endDate, 'startDateIsBeforeEndDate')
    }
    return null;
}

export function removeErrorFromField(field: AbstractControl, error: string) {
    if (field.errors) {
        delete field.errors[error];
    }
    if (JSON.stringify(field.errors) == JSON.stringify({})) {
        field.setErrors(null)
    }
}

export function startDateIsBeforeEndDateValidator(startDate: AbstractControl, endDate: AbstractControl): ValidationErrors | null {
    if(startDate && endDate && startDate.value && endDate.value) {
        const forbidden = moment(startDate.value).isSameOrAfter(endDate.value) ? { startDateIsBeforeEndDate: { value: endDate.value } } : null;
        if(forbidden) {
            startDate.markAsDirty();
            endDate.markAsDirty();
            startDate.setErrors(forbidden);
            endDate.setErrors(forbidden);
            return forbidden;
        }
        removeErrorFromField(endDate, 'startDateIsBeforeEndDate');
        removeErrorFromField(startDate, 'startDateIsBeforeEndDate');
    } else if (startDate && startDate.value && !(endDate && endDate.value)) {
        removeErrorFromField(startDate, 'startDateIsBeforeEndDate')
    }
    else if (!(startDate && startDate.value) && endDate && endDate.value) {
        removeErrorFromField(endDate, 'startDateIsBeforeEndDate')
    }
    return null;
}
