import { FieldType } from "@seniorsistemas/angular-components";

const numberTypes = [FieldType.Integer, FieldType.Double];
const numberPattern = /^\d+\b$/;

export const mergeUnique = (array: Array<string>, otherArray: Array<string>) =>
    array.concat(otherArray.filter(item => !array.includes(item)));

export const isValidFilter = (type: FieldType, value: any) => {
    const { isValidValue, typeExists } = getTypeInformation(type, value);
    return !(typeExists && !isValidValue);
};

export const getTypeInformation = (type: FieldType, value: any) => {
    switch (type) {
        case FieldType.Time:
        case FieldType.Date:
        case FieldType.DateTime:
            return { isValidValue: false, typeExists: true };
        case FieldType.Integer:
        case FieldType.Double:
            return {
                isValidValue: value.match(numberPattern),
                typeExists: numberTypes.includes(type),
            };
        default:
            return {
                isValidValue: value.match(numberPattern),
                typeExists: numberTypes.includes(type),
            };
    }
};

export const convertMinutesToTime = (minutes: string, monthMinute?: boolean) => {
    let time = minutes.replace('.', '');
    let hour = Math.floor(parseInt(time) / 60)
    let minute = parseInt(time) % 60
    let msg = (hour < 10) ? '0' + hour : hour;
    msg += ':'
    msg += (minute < 10) ? '0' + minute : '' + minute;
    if (monthMinute) {
        msg = putZeroAtHours(msg.toString());
    }
    return msg;
};

const putZeroAtHours = (msg: string) => {
    if (msg.length === 4) {
        return '00' + msg
    } else if (msg.length === 5) {
        return '0' + msg
    } else {
        return msg;
    }
};

export const convertTimeToMinutes = (time: any) => {
    if (time) {
        const array = time.toString().split(':');
        return (array[0] * 60) + parseInt(array[1]);
    }
    return null;
};