import { Component, Injectable, NgModule } from "@angular/core";
import { ActivatedRouteSnapshot, CanActivate, Router, RouterModule, Routes } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { LocaleService } from "@seniorsistemas/angular-components";
import { PermissionsService } from "@seniorsistemas/platform-components";
import { map } from "rxjs/operators";
/*{CA:PACKAGE_IMPORTS:START}*/
/*{CA:PACKAGE_IMPORTS:END}*/

import { BridgeService } from "@senior-gestao-pessoas/hcm-clocking-event-foundation-schedules-histories";
import { BridgeShowcaseComponent } from "./bridge-showcase.component";

/*{CA:PROJECT_IMPORTS:START}*/
/*{CA:PROJECT_IMPORTS:END}*/

@Injectable()
export class BridgeFeatureRoutingGuard implements CanActivate {
    constructor(private permissionsService: PermissionsService, private router: Router) { }

    public canActivate() {
        return this.permissionsService.get("bridge").pipe(
            map((permissions: any) => {
                if (!permissions.visualizar) this.router.navigate(["forbidden"]);
                return permissions.visualizar;
            })
        );
    }
}

@Injectable()
export class BridgeFeatureRoutingPermissionResolver {
    constructor(private permissionsService: PermissionsService) { }

    public resolve() {
        return this.permissionsService.get();
    }
}

@Injectable()
export class BridgeFeatureRoutingLocaleResolver {
    constructor(private localeService: LocaleService) { }

    public resolve() {
        return this.localeService.get();
    }
}

@Injectable()
export class BridgeFeatureRoutingEntityResolver {
    constructor(private bridgeService: BridgeService) { }

    public resolve(route: ActivatedRouteSnapshot) {
        if (route.params.bridge === "new") return;

        return this.bridgeService.get(route.params.bridge);
    }
}

@Component({
    template: `<router-outlet></router-outlet>`,
})
export class EmptyComponent { }

export const routes: Routes = [
    {
        path: "bridge",
        component: EmptyComponent,
        canActivate: [
            BridgeFeatureRoutingGuard,
            /*{CA:BADGE_HISTORY_ROUTE_GUARDS:START}*/
            /*{CA:BADGE_HISTORY_ROUTE_GUARDS:END}*/
        ],
        resolve: {
            allPermissions: BridgeFeatureRoutingPermissionResolver,
            localeConfig: BridgeFeatureRoutingLocaleResolver,
            /*{CA:BADGE_HISTORY_ROUTE_RESOLVE:START}*/
            /*{CA:BADGE_HISTORY_ROUTE_RESOLVE:END}*/
        },
        children: [
            {
                path: "",
                component: BridgeShowcaseComponent,
            },
            {
                path: ":bridge",
                component: EmptyComponent,
                canActivate: [BridgeFeatureRoutingGuard],
                resolve: {
                    allPermissions: BridgeFeatureRoutingPermissionResolver,
                    localeConfig: BridgeFeatureRoutingLocaleResolver,
                    entity: BridgeFeatureRoutingEntityResolver,
                    /*{CA:BADGE_HISTORY_ROUTE_RESOLVE:START}*/
                    /*{CA:BADGE_HISTORY_ROUTE_RESOLVE:END}*/
                },
            },
            /*{CA:BADGE_HISTORY_ROUTE_CHILDREN:START}*/
            /*{CA:BADGE_HISTORY_ROUTE_CHILDREN:END}*/
        ],
    },
];

@NgModule({
    imports: [
        RouterModule.forChild(routes),
        /*{CA:MODULE_IMPORTS:START}*/
        /*{CA:MODULE_IMPORTS:END}*/
    ],
    exports: [
        RouterModule,
        /*{CA:MODULE_EXPORTS:START}*/
        /*{CA:MODULE_EXPORTS:END}*/
    ],
    providers: [
        BridgeFeatureRoutingGuard,
        BridgeFeatureRoutingPermissionResolver,
        BridgeFeatureRoutingLocaleResolver,
        BridgeFeatureRoutingEntityResolver,
        /*{CA:MODULE_PROVIDERS:START}*/
        /*{CA:MODULE_PROVIDERS:END}*/
    ],
    declarations: [
        EmptyComponent,
        /*{CA:MODULE_DECLARATIONS:START}*/
        /*{CA:MODULE_DECLARATIONS:END}*/
    ],
    /*{CA:MODULE_CONFIG:START}*/
    /*{CA:MODULE_CONFIG:END}*/
})
export class BridgeShowcaseRoutingModule {
}

/*{CA:FILE_CONTENTS:START}*/
/*{CA:FILE_CONTENTS:END}*/
